import { Task, } from './task';
export var AgentStatus;
(function (AgentStatus) {
    AgentStatus["Online"] = "online";
    AgentStatus["Offline"] = "offline";
    AgentStatus["Pause"] = "pause";
    AgentStatus["BreakOut"] = "break_out";
})(AgentStatus || (AgentStatus = {}));
export var ChannelState;
(function (ChannelState) {
    ChannelState["Waiting"] = "waiting";
    ChannelState["Distribute"] = "distribute";
    ChannelState["Offering"] = "offering";
    ChannelState["Answered"] = "answered";
    ChannelState["Active"] = "active";
    ChannelState["Bridged"] = "bridged";
    ChannelState["Hold"] = "hold";
    ChannelState["Missed"] = "missed";
    ChannelState["WrapTime"] = "wrap_time";
    ChannelState["Processing"] = "processing";
    ChannelState["Transfer"] = "transfer";
})(ChannelState || (ChannelState = {}));
export var ChannelType;
(function (ChannelType) {
    ChannelType["Call"] = "call";
    ChannelType["Email"] = "email";
    ChannelType["Chat"] = "chat";
})(ChannelType || (ChannelType = {}));
export class Agent {
    constructor(client, info) {
        this.client = client;
        this.info = info;
        this.task = new Map();
        this._channel = info.channels[0]; // todo
        this.lastStatusChange = Date.now() - this.info.status_duration * 1000;
    }
    get agentId() {
        return this.info.agent_id;
    }
    get onDemand() {
        return this.info.on_demand;
    }
    get status() {
        return this.info.status;
    }
    get statusPayload() {
        return this.info.status_payload;
    }
    get isAdmin() {
        return this.info.is_admin;
    }
    get isSupervisor() {
        return this.info.is_supervisor;
    }
    get team() {
        return this.info.team;
    }
    get supervisor() {
        return this.info.supervisor;
    }
    get auditor() {
        return this.info.auditor;
    }
    get channels() {
        return [this._channel];
    }
    get channel() {
        return this._channel;
    }
    get stateDuration() {
        return Math.round((Date.now() - this.lastStatusChange) / 1000);
    }
    // todo need refactor
    onChannelEvent(e) {
        let task;
        switch (e.status) {
            case ChannelState.Distribute:
                const distributeEvent = e;
                if (!distributeEvent) {
                    throw new Error('bad event');
                }
                task = new Task(this.client, e, distributeEvent.distribute);
                this.task.set(task.id, task);
                if (task.agentChannelId) {
                    switch (task.channel) {
                        case ChannelType.Call:
                            const call = this.client.callById(task.agentChannelId);
                            if (call && !call.task) {
                                call.task = task;
                            }
                            break;
                        case ChannelType.Chat:
                            const chat = this.client.conversationById(task.agentChannelId);
                            if (chat && !chat.task) {
                                chat.task = task;
                            }
                            break;
                        default:
                    }
                }
                break;
            case ChannelState.Offering:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.offeringAt = e.timestamp;
                }
                break;
            case ChannelState.Bridged:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.bridgedAt = e.timestamp;
                }
                break;
            case ChannelState.Transfer:
                const transfer = e;
                task = this.task.get(transfer.to_attempt_id);
                if (task) {
                    this.task.delete(task.id);
                    task.id = transfer.attempt_id;
                    task.setTransferred(transfer.distribute);
                    this.task.set(task.id, task);
                }
                break;
            case ChannelState.Answered:
                task = this.task.get(e.attempt_id);
                if (task) {
                    task.answeredAt = e.timestamp;
                }
                break;
            case ChannelState.Missed:
                if (e.attempt_id) {
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.stopAt = e.timestamp;
                        const missedEvent = e;
                        if (!missedEvent) {
                            throw new Error('bad event');
                        }
                        this.setChannelStateTimeout(e.channel, e, missedEvent.missed.timeout);
                        this.task.delete(e.attempt_id);
                        this.client.reportingChannelTask(task);
                        return task;
                    }
                }
                break;
            case ChannelState.WrapTime:
                if (e.attempt_id) {
                    const wrapTimeEvent = e;
                    if (!wrapTimeEvent) {
                        throw new Error('bad event');
                    }
                    task = this.task.get(e.attempt_id);
                    this.task.delete(e.attempt_id);
                    this.client.reportingChannelTask(task);
                }
                break;
            case ChannelState.Processing:
                if (e.attempt_id) {
                    const processingEvent = e;
                    if (!processingEvent) {
                        throw new Error('bad event');
                    }
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.state = e.status;
                        task.closedAt = e.timestamp; // todo
                        task.setProcessing(processingEvent.processing);
                        this.setChannelStateTimeout(e.channel, e, processingEvent.processing.timeout);
                        return task;
                    }
                }
                break;
            // TODO
            case ChannelState.Waiting:
                if (e.attempt_id) {
                    task = this.task.get(e.attempt_id);
                    if (task) {
                        task.stopAt = e.timestamp;
                        this.task.delete(e.attempt_id);
                        this.client.reportingChannelTask(task);
                    }
                }
                break;
            default:
            // throw new Error("not found task")
        }
        this.setChannelState(e.channel, e);
        if (task) {
            task.state = e.status;
            return task;
        }
        else {
            return undefined;
        }
    }
    async online(channels, onDemand) {
        return this.client.agentSetOnline(this.agentId, channels, onDemand);
    }
    async waiting(channel) {
        return this.client.agentSetWaiting(this.agentId, channel);
    }
    async pause(payload) {
        return this.client.agentSetPause(this.agentId, payload);
    }
    async offline() {
        return this.client.agentSetOffline(this.agentId);
    }
    setStatus(e) {
        if (e.status === AgentStatus.Online) {
            this.info.on_demand = e.on_demand || false;
            this._channel = e.channels[0];
        }
        this.info.status = e.status;
        this.info.status_payload = e.status_payload ? e.status_payload : null;
        this.lastStatusChange = Date.now();
    }
    async directMember(memberId, communicationId) {
        return this.client.request(`cc_member_direct`, {
            agent_id: this.agentId,
            member_id: memberId,
            communication_id: communicationId,
        });
    }
    async offlineMembers(q, page, perPage) {
        return this.client.request(`cc_fetch_offline_members`, {
            agent_id: this.agentId,
            page,
            q,
            per_page: perPage,
        });
    }
    hasTask(task) {
        return this.task.has(task.id);
    }
    setChannelState(name, e) {
        this._channel.channel = name ? name : null;
        this._channel.state = e.status;
        this._channel.joined_at = e.timestamp;
        this._channel.timeout = undefined;
    }
    setChannelStateTimeout(name, e, timeout) {
        this._channel.channel = name ? name : null;
        this._channel.state = e.status; // FIXME
        this._channel.timeout = timeout;
        this._channel.joined_at = e.timestamp;
    }
}
//# sourceMappingURL=agent.js.map