import { __awaiter, __generator } from "tslib";
import { chunkString } from './utils';
var maxSizeMessage = 4096;
export var ChatActions;
(function (ChatActions) {
    ChatActions["Message"] = "message";
    ChatActions["UserInvite"] = "user_invite";
    ChatActions["Invite"] = "invite_conversation";
    ChatActions["Joined"] = "join_conversation";
    ChatActions["Close"] = "close_conversation";
    ChatActions["Leave"] = "leave_conversation";
    ChatActions["Decline"] = "decline_invite";
    ChatActions["Update"] = "update_channel";
    ChatActions["Destroy"] = "destroy";
})(ChatActions || (ChatActions = {}));
export var ConversationState;
(function (ConversationState) {
    ConversationState["Init"] = "init";
    ConversationState["Invite"] = "invite";
    ConversationState["Active"] = "active";
    ConversationState["Pending"] = "pending";
    ConversationState["Closed"] = "closed";
})(ConversationState || (ConversationState = {}));
var Conversation = /** @class */ (function () {
    function Conversation(client, conversationId, title, members, messages, variables) {
        this.client = client;
        this.conversationId = conversationId;
        this.title = title;
        this.channelId = null;
        this.createdAt = Date.now();
        this.answeredAt = 0;
        this.updatedAt = 0;
        this.invitedAt = 0;
        this.closedAt = 0;
        this.task = null;
        this.data = null;
        this.members = (members || []).map(function (i) { return wrapChannelMember(i); });
        this._messages = messages || [];
        this.state = ConversationState.Invite;
        this.variables = variables;
        if (variables &&
            variables.hasOwnProperty('cc_attempt_id') &&
            this.client.agent) {
            this.task = this.client.agent.task.get(+variables.cc_attempt_id) || null;
        }
    }
    Conversation.prototype.setInvite = function (inviteId, timestamp) {
        this.inviteId = inviteId;
        this.invitedAt = timestamp;
    };
    Conversation.prototype.setAnswered = function (channelId, timestamp, member) {
        this.state = ConversationState.Active;
        this.answeredAt = timestamp;
        this.channelId = channelId;
        this.member = wrapChannelMember(member);
        this.inviteId = null; // deleted in DB
    };
    Conversation.prototype.setClosed = function (timestamp) {
        this.state = ConversationState.Closed;
        this.closedAt = timestamp;
    };
    Object.defineProperty(Conversation.prototype, "id", {
        get: function () {
            return this.conversationId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Conversation.prototype, "messages", {
        get: function () {
            return this.getMessages();
        },
        enumerable: true,
        configurable: true
    });
    Conversation.prototype.getMessages = function () {
        var _this = this;
        return this._messages.map(function (i) {
            var msg = {
                id: i.id,
                type: i.type,
                member: _this.messageMember(i),
                channelId: i.channel_id,
                createdAt: i.created_at,
                updatedAt: i.updated_at,
            };
            if (i.hasOwnProperty('file')) {
                i.file.url = _this.client.fileUrlDownload(i.file.id);
                msg.file = i.file;
            }
            if (i.hasOwnProperty('text')) {
                msg.text = i.text;
            }
            return msg;
        });
    };
    Conversation.prototype.next = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error('TODO');
            });
        });
    };
    Conversation.prototype.pagination = function (page, perPage) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                throw new Error('TODO');
            });
        });
    };
    Conversation.prototype.newMessage = function (e) {
        this._messages.push(e);
    };
    Object.defineProperty(Conversation.prototype, "allowDecline", {
        get: function () {
            return !!this.inviteId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Conversation.prototype, "allowJoin", {
        get: function () {
            return !!this.inviteId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Conversation.prototype, "allowLeave", {
        get: function () {
            return !!this.channelId && this.closedAt === 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Conversation.prototype, "allowReporting", {
        get: function () {
            return this.answeredAt > 0 && this.hasReporting;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Conversation.prototype, "hasReporting", {
        get: function () {
            return this.variables && this.variables.cc_reporting === 'true';
        },
        enumerable: true,
        configurable: true
    });
    /*
    Actions
     */
    Conversation.prototype.decline = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.inviteId)
                    throw new Error('This conversation is joined');
                return [2 /*return*/, this.client.request("decline_chat", {
                        invite_id: this.inviteId,
                    })];
            });
        });
    };
    Conversation.prototype.join = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.inviteId)
                    throw new Error('This conversation is joined');
                return [2 /*return*/, this.client.request("join_chat", {
                        invite_id: this.inviteId,
                    })];
            });
        });
    };
    Conversation.prototype.close = function (cause) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.channelId)
                    throw new Error('This conversation not active');
                return [2 /*return*/, this.client.request("close_chat", {
                        channel_id: this.channelId,
                        conversation_id: this.id,
                        cause: cause,
                    })];
            });
        });
    };
    Conversation.prototype.leave = function (cause) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.channelId)
                    throw new Error('This conversation not active');
                return [2 /*return*/, this.client.request("leave_chat", {
                        channel_id: this.channelId,
                        conversation_id: this.id,
                        cause: cause,
                    })];
            });
        });
    };
    Conversation.prototype.sendText = function (text) {
        return __awaiter(this, void 0, void 0, function () {
            var chunk, i;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.channelId) {
                            throw new Error('conversation not active');
                        }
                        if (!text.length) {
                            throw new Error('empty message');
                        }
                        chunk = chunkString(text, maxSizeMessage);
                        i = 0;
                        _a.label = 1;
                    case 1:
                        if (!(i < chunk.length)) return [3 /*break*/, 4];
                        if (i + 1 >= chunk.length) {
                            return [2 /*return*/, this.sendMessageTextChunk(chunk[i])];
                        }
                        return [4 /*yield*/, this.sendMessageTextChunk(chunk[i])];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        i++;
                        return [3 /*break*/, 1];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    Conversation.prototype.sendFile = function (file, cb) {
        return __awaiter(this, void 0, void 0, function () {
            var storedFiles, f;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.client.storeFile(this.id, [file], cb)];
                    case 1:
                        storedFiles = _a.sent();
                        f = storedFiles[0];
                        // todo bug if chat response error
                        return [2 /*return*/, this.client.request("send_file_chat", {
                                channel_id: this.channelId,
                                conversation_id: this.id,
                                id: f.id,
                                name: file.name,
                                mime: f.mime,
                                url: f.shared,
                                size: f.size,
                            })];
                }
            });
        });
    };
    Conversation.prototype.send = function (data, cb) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (typeof data === 'string') {
                    return [2 /*return*/, this.sendText(data)];
                }
                else if (data instanceof File) {
                    return [2 /*return*/, this.sendFile(data, cb)];
                }
                else {
                    throw new Error('unknown send data');
                }
                return [2 /*return*/];
            });
        });
    };
    Conversation.prototype.addToChat = function (userId, title) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("add_to_chat", {
                        channel_id: this.channelId,
                        conversation_id: this.id,
                        user_id: userId,
                        title: title,
                    })];
            });
        });
    };
    Conversation.prototype.startChat = function (userId) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("start_chat", {
                        user_id: userId,
                    })];
            });
        });
    };
    Conversation.prototype.updateChannel = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("update_channel_chat", {
                        channel_id: this.channelId,
                    })];
            });
        });
    };
    Conversation.prototype.reporting = function (reporting) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.task) {
                    throw new Error("this conversation not in queue");
                }
                return [2 /*return*/, this.task.reporting(reporting)];
            });
        });
    };
    Conversation.prototype.renew = function (sec) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                if (!this.task) {
                    throw new Error("this conversation not in queue");
                }
                return [2 /*return*/, this.task.renew(sec)];
            });
        });
    };
    Conversation.prototype.sendMessageTextChunk = function (text) {
        return this.client.request("send_text_chat", {
            channel_id: this.channelId,
            conversation_id: this.id,
            text: text,
        });
    };
    Conversation.prototype.messageMember = function (msg) {
        if (this.member && msg.channel_id === this.channelId) {
            return {
                self: true,
            };
        }
        for (var _i = 0, _a = this.members; _i < _a.length; _i++) {
            var m = _a[_i];
            if (m.id === msg.channel_id) {
                return m;
            }
        }
        return null; // TODO
    };
    return Conversation;
}());
export { Conversation };
function wrapChannelMember(m) {
    // todo
    m.messenger = m.type;
    return m;
}
//# sourceMappingURL=conversation.js.map