import { __assign, __awaiter, __generator } from "tslib";
export var ChannelName;
(function (ChannelName) {
    ChannelName["Call"] = "call";
    ChannelName["Chat"] = "chat";
    ChannelName["Task"] = "task";
})(ChannelName || (ChannelName = {}));
var Task = /** @class */ (function () {
    function Task(client, e, distribute) {
        this.client = client;
        this.distribute = distribute;
        this.id = e.attempt_id;
        this.state = e.status;
        this.lastStatusChange = e.timestamp;
        this.createdAt = e.timestamp;
        this._processing = null;
        this.offeringAt = 0;
        this.answeredAt = 0;
        this.bridgedAt = 0;
        this.startProcessingAt = 0;
        this.stopAt = 0;
        this.closedAt = 0;
        this.communication = distribute.communication;
        this.history = [distribute];
    }
    Object.defineProperty(Task.prototype, "queue", {
        get: function () {
            return {
                id: this.distribute.queue_id,
                name: this.distribute.queue_name,
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "duration", {
        get: function () {
            return Math.round((Date.now() - this.lastStatusChange) / 1000);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "channel", {
        get: function () {
            return this.distribute.channel;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "allowAccept", {
        get: function () {
            return (this.channel === ChannelName.Task &&
                (this.bridgedAt === 0 && this.closedAt === 0));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "allowDecline", {
        get: function () {
            return this.allowAccept;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "allowClose", {
        get: function () {
            return (this.channel === ChannelName.Task &&
                this.closedAt === 0 &&
                this.bridgedAt > 0);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "memberId", {
        get: function () {
            return this.distribute.member_id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "queueId", {
        get: function () {
            return this.distribute.queue_id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "hasReporting", {
        get: function () {
            return this.distribute.has_reporting;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "allowReporting", {
        get: function () {
            return this.hasReporting && this.bridgedAt > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "agentChannelId", {
        get: function () {
            return this.distribute.agent_channel_id;
        },
        enumerable: true,
        configurable: true
    });
    Task.prototype.setAnswered = function (t) {
        this.answeredAt = t;
        this.lastStatusChange = Date.now();
    };
    Task.prototype.setBridged = function (t) {
        this.bridgedAt = t;
        this.lastStatusChange = Date.now();
    };
    Task.prototype.setProcessing = function (p) {
        if (!this.startProcessingAt) {
            this.startProcessingAt = Date.now();
        }
        if (p.sec) {
            p.timeout = Date.now() + p.sec * 1000; // bug
        }
        this._processing = p;
    };
    Task.prototype.setTransferred = function (d) {
        this.distribute = d;
        this.history.push(d);
    };
    Object.defineProperty(Task.prototype, "processingTimeoutAt", {
        get: function () {
            if (!this._processing || !this._processing.timeout) {
                return null;
            }
            return this._processing.timeout;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "processingSec", {
        get: function () {
            if (!this._processing || !this._processing.sec) {
                return null;
            }
            return this._processing.sec;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Task.prototype, "renewalSec", {
        get: function () {
            if (!this._processing || !this._processing.renewal_sec) {
                return null;
            }
            return this._processing.renewal_sec;
        },
        enumerable: true,
        configurable: true
    });
    /*
      control
     */
    Task.prototype.accept = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("cc_agent_task_accept", {
                        agent_id: this.distribute.agent_id,
                        attempt_id: this.id,
                        app_id: this.distribute.app_id,
                    })];
            });
        });
    };
    Task.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("cc_agent_task_close", {
                        agent_id: this.distribute.agent_id,
                        attempt_id: this.id,
                        app_id: this.distribute.app_id,
                    })];
            });
        });
    };
    // todo
    Task.prototype.decline = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request("cc_agent_task_close", {
                        agent_id: this.distribute.agent_id,
                        attempt_id: this.id,
                        app_id: this.distribute.app_id,
                    })];
            });
        });
    };
    Task.prototype.reporting = function (reporting) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request('cc_reporting', __assign({ attempt_id: this.id }, reporting))];
            });
        });
    };
    Task.prototype.renew = function (sec) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.client.request('cc_renewal', {
                        attempt_id: this.id,
                        renewal_sec: sec ? sec : this.processingSec,
                    })];
            });
        });
    };
    return Task;
}());
export { Task };
//# sourceMappingURL=task.js.map